### Name: bvls
### Title: The Stark-Parker algorithm for bounded-variable least squares
### Aliases: bvls
### Keywords: optimize

### ** Examples


## simulate a matrix A
## with 3 columns, each containing an exponential decay 
t <- seq(0, 2, by = .04)
k <- c(.5, .6, 1)
A <- matrix(nrow = 51, ncol = 3)
Acolfunc <- function(k, t) exp(-k*t)
for(i in 1:3) A[,i] <- Acolfunc(k[i],t)

## simulate a matrix X
X <- matrix(nrow = 51, ncol = 3) 
wavenum <- seq(18000,28000, by=200)
location <- c(25000, 22000) 
delta <- c(1000,1000)
Xcolfunc <- function(wavenum, location, delta)
  exp( - log(2) * (2 * (wavenum - location)/delta)^2)
for(i in 1:2) X[,i] <- Xcolfunc(wavenum, location[i], delta[i])

X[1:40,3] <- Xcolfunc(wavenum, 23000, 1000)[12:51]
X[41:51,3]<- - Xcolfunc(wavenum, 23000, 1000)[21:31]

## set seed for reproducibility
set.seed(3300)

## simulated data is the product of A and X with some
## spherical Gaussian noise added 
matdat <- A %*% t(X) + .01 * rnorm(nrow(A) * nrow(X))

## estimate the rows of X using BVLS criteria 
bvls_sol <- function(matdat, A) {
  X <- matrix(0, nrow = 51, ncol = 3)
  bu <- rep(1000,3)
  bl <- c(0,0,-1000)
  for(i in 1:ncol(matdat)) 
     X[i,] <- coef(bvls(A,matdat[,i], bl, bu))
  X
}
X_bvls <- bvls_sol(matdat,A) 

## Not run:  
##D ## can solve the same problem with L-BFGS-B algorithm
##D ## but need starting values for x 
##D bfgs_sol <- function(matdat, A) {
##D   startval <- rep(0, ncol(A))
##D   fn1 <- function(par1, b, A) sum( ( b - A %*% par1)^2)
##D   X <- matrix(0, nrow = 51, ncol = 3)
##D   bu <- rep(1000,3)
##D   bl <- c(0,0,-1000)
##D   for(i in 1:ncol(matdat))  
##D     X[i,] <-  optim(startval, fn = fn1, b=matdat[,i], A=A,
##D                   upper = bu, lower = bl,
##D                   method="L-BFGS-B")$par
##D     X
##D }
##D X_bfgs <- bfgs_sol(matdat,A) 
##D 
##D ## the RMS deviation under NNLS is less than under L-BFGS-B 
##D sqrt(sum((X - X_bvls)^2)) < sqrt(sum((X - X_bfgs)^2))
##D 
##D ## and L-BFGS-B is much slower 
##D system.time(bvls_sol(matdat,A))
##D system.time(bfgs_sol(matdat,A))
## End(Not run)



